/*
 * grav.c: Calculate the gravitational force and potential for
 * particle at pos0 due to list of particles.
 *
 * Michael S. Warren, Sep. 11, 1995.  mswarren@lanl.gov
 */

#include <math.h>
#include "grav.h"
#include "vop.h"

void 
do_grav(const float *p, const float *end, const float *pos0, float *mass0, 
	float *acc0, float *phi0, const float *eps2p)
{
    float dr2;
    float r[NDIM];
    float phii, mor3, mass;
    const float ppos[NDIM] = {pos0[0], pos0[1], pos0[2]};
    float a[NDIM];
    float phi = *phi0;
    float total_mass = *mass0;
    const float eps2 = *eps2p;

    VV(a, = acc0);

    while (p < end) {
	mass = *p++;
	r[0] = *p++;
	r[1] = *p++;
	r[2] = *p++;
	VV(r, -= ppos);

	dr2 = Dot(r, r);
	dr2 += eps2;		/* force smoothing */

	phii = 1.0/sqrt(dr2);
	
	mor3 = phii * phii;
	phii *= mass;
	total_mass += mass;
	mor3 *= phii;
	phi -= phii;

	VV(a, += mor3 * r);
    }
    VV(acc0, = a);
    *mass0 = total_mass;
    *phi0 = phi;
}
