/*
 * utils.c: Various helpful functions.
 *
 * Michael S. Warren, Sep. 11, 1995.  mswarren@lanl.gov
 */
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <mpi.h>
#include "utils.h"
#include "rand.h"

void
Error(const char * mesg, ...)	/* print error message and die */
{
    int procnum;
    va_list alist;

    MPI_Comm_rank(MPI_COMM_WORLD, &procnum);
    va_start(alist, mesg);
    fprintf(stderr, "ERROR: Node %d ", procnum);
    vfprintf(stderr, mesg, alist);
    fflush(stderr);
    va_end(alist);
    MPI_Abort(MPI_COMM_WORLD, 1);
}

void *
Malloc(unsigned int size)	/* Wrapper to check for malloc failure */
{
    void *ptr;

    if (size == 0) {
	return (void *)0;
    }
    ptr = malloc(size);
    if (ptr == (void *)0) {
	Error("Malloc(%d) failed\n", size);
    }
    return(ptr);
}

void
Free(void *ptr)
{
    free(ptr);
}

void 
singlPrintf(const char *fmt, ...)     /* Print only if we are proc 0 */
{
    int procnum;
    va_list ap;

    MPI_Comm_rank(MPI_COMM_WORLD, &procnum);

    if (procnum != 0)
      return;
    va_start(ap, fmt);
    vfprintf(stdout, fmt, ap);
    va_end(ap);
    fflush(stdout);
}

float
sphere_rand(int ndim, float *x)	  /* Generate a random vector with norm < 1 */
{
    int k;
    float rsqx;
 
    do {
	rsqx = (float)0.0;
	for (k = 0; k < ndim; k++) {
            x[k] = uniform_rand()*2.0F - 1.0F; /* a pt in (-1,1) */
            rsqx += x[k] * x[k];
	}
    } while (rsqx > (float)1.0);
    return rsqx;
}
